%% Market data analysis GUI
function gui
    
    %% Create main figure
    delete(findall(0,'type','figure', 'Name','Market analysis'));
    fig = figure('Name','Market analysis', 'NumberTitle','off', 'Position',[50,100,700,450], 'Toolbar','figure');
    
    %% Fix the toolbar's Open button
    updateOpenFile(fig);

end

%% Fix the toolbar's Open button
function updateOpenFile(fig)
    % Get handle and properties of the toolbar's <Open File> button
    hOpen = findall(fig,'tooltipstring','Open File');  % or: 'tag','Standard.FileOpen'
    open_data = get(hOpen);
    hToolbar = open_data.Parent;

    % Hide the existing open button
    delete(hOpen);

    % Create a new split-button with the open-file button's data
    hNewOpen = uisplittool('Parent',hToolbar, ...
                           'CData',open_data.CData, ...
                           'Tooltip',open_data.TooltipString, ...
                           'ClickedCallback',@openFile);

    % Move the button to the very left of the toolbar
    drawnow;
    jToolbar = get(get(hToolbar,'JavaContainer'),'ComponentPeer');
    jButtons = jToolbar.getComponents;
    jToolbar.setComponentZOrder(jButtons(end),0);
    jToolbar.revalidate;

    % Get the initial list of pre-defied files
    files = dir('*.csv');
    filenames = {files.name};

    % Update the list of recent files in the main menu and the toolbar's split-button
    jNewOpen = get(hNewOpen,'JavaContainer');
    jNewOpenMenu = jNewOpen.getMenuComponent;
    for fileIdx = 1 : length(filenames)
        jMenuItem = handle(jNewOpenMenu.add(filenames{fileIdx}),'CallbackProperties');
        set(jMenuItem,'ActionPerformedCallback',{@openFile,filenames{fileIdx}});
    end
    
    % Now update the File main menu:
    hFileMenu = findall(gcf, 'tag', 'figMenuFile');
    hMruMenu = uimenu('Label','Recent files', 'Parent',hFileMenu);
    hAllMenuItems = allchild(hFileMenu);
    set(hFileMenu, 'Children',fliplr(hAllMenuItems([2:end-1,1,end])));  % place in 2nd position, just above the "Open" item

    % Update the Open... menu item
    hOpenMenu = findall(hFileMenu, 'Label', '&Open...');
    set(hOpenMenu, 'Callback',@openFile);
    
    % Add the list of recent files, one item at a time
    for fileIdx = 1 : length(filenames)
        uimenu(hMruMenu, 'Label',filenames{fileIdx}, 'Callback',{@openFile,filenames{fileIdx}});
    end
end

%% Callback for the open-file toolbar button
function openFile(hObject,eventData,filename) %#ok<INUSL>
    % If no filename specified, ask for it from the user
    if nargin < 3
        filename = uigetfile({'*.csv','Data files (*.csv)'},'Open data file');
        if isempty(filename) || isequal(filename,0)
            return;
        end
    end
   
    % Open the filename and update the data
    fid = fopen(filename);
    data = textscan(fid,'%s%f%f%f%f%f%f',-1, 'delimiter',',', 'whitespace','\b\t', ...
                    'headerlines',1, 'headercolumns',0,...
                    'returnonerror',0, 'emptyvalue',0, 'CollectOutput', true);
    fclose(fid);
    
    % Fix the date column
    data{1} = datestr(datenum(data{1},'dd/mm/yyyy'),'yyyy-mm-dd');
    data{1} = mat2cell(data{1}, ones(size(data{1},1),1), 10);
    
    % Add change columns
    data{2}(:,end+1) = round(100*  (data{2}(:,4) -  data{2}(:,1)))/100;     % absolute change
    data{2}(:,end+1) = round(10000*(data{2}(:,4) ./ data{2}(:,1) -1))/100;  % relative change
    
    % Concatenate into one large cell array (matrix)
    data = [data{1} num2cell(data{2})];
    
    % Update the display
    updateGUI(data,filename)
end

%% Update the GUI with newly-loaded data
function updateGUI(data,filename)
    
    %% Clear the figure and set a new parent panel
    clf;
    hPanel = uipanel('Units','normalized', 'Position',[0,0,1,1], 'BorderType','none');

    %% Present the title
    [fpath,fname] = fileparts(filename); %#ok<ASGLU>
    hTitle = uicontrol('Style','text', 'String',['Historical data for ' upper(fname)], ...
                       'Units','normalized', 'Position',[.2,.95,.6,.04], 'Parent',hPanel, ...
                       'FontWeight','bold', 'FontSize',12);
    drawnow;

    %% enable changing title font
    allFonts = listfonts;
    currentFont = find(strcmpi(get(hTitle,'FontName'),allFonts),1);
    if isempty(currentFont),  currentFont=1;  end
    fontStr = @(fontName) ['<html><font face="' fontName '">' fontName];
    htmlStr = cellfun(fontStr, allFonts, 'uniform',false);
    hFont = uicontrol('Style','popup', 'String',htmlStr, 'value',currentFont, ...
                      'Callback',{@updateFont,hTitle}, ...
                      'Units','normalized', 'Position',[0.75,.95,0.18,.04], 'Parent',hPanel); %#ok<NASGU>
                   
    %% Start with a plain data-table
    warning off MATLAB:uitable:OldTableUsage
    warning off MATLAB:uitable:DeprecatedFunction
    headers = {'Date','Open','High','Low','Close','Volume','Adj Close','Change','%Change'};
    hTable = uitable('Data',data, 'ColumnNames',headers, 'Parent',hPanel);
    set(hTable, 'Units','normalized', 'Position',[.02,.02,.95,.9]);
    drawnow;
    
    %% Make the headers bold-blue
    headers = strcat('<html><b><font color="blue">', headers);
    set(hTable, 'ColumnNames',headers);
    
    %% Make the columns auto-resize
    jTable = hTable.getTable;
    jTable.setAutoResizeMode(jTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS)
    drawnow;
    
    %% Add the start-date selector combo
    startDate = java.util.Date(datestr(datenum(data{end,1})));
    com.mathworks.mwswing.MJUtilities.initJIDE;
    jDateChooser = handle(com.jidesoft.combobox.DateComboBox, 'CallbackProperties');
    jDateChooser.setDate(startDate);
    jDateChooser.setToolTipText('Set the earliest display date');

    % Set the min date
    calendar = java.util.Calendar.getInstance;
    calendar.setTime(startDate);
    jDateChooser.getDateModel.setMinDate(calendar);

    % Set the selection callback
    set(jDateChooser, 'ItemStateChangedCallback', {@updateDate,hTable,data});

    % Place on-screen
    [jhDateChooser,hContainer] = javacomponent(jDateChooser,[],hPanel); %#ok<ASGLU>
    set(hContainer, 'Units','normalized', 'Position',[.05,.94,.15,.05]);
    
    %% Hide the vertical scroll-bar when not needed
    jScroll = hTable.TableScrollPane;
    jScroll.setVerticalScrollBarPolicy(jScroll.VERTICAL_SCROLLBAR_AS_NEEDED);
    drawnow;

    % Enable cell rendering customizations
    javaaddpath('ColoredFieldCellRenderer.zip');
    cr = ColoredFieldCellRenderer;
    for col = 1 : size(data,2)
        jTable.getColumnModel.getColumn(col-1).setCellRenderer(cr);
    end
    
    %% Color dates with strong price changes
    numRows = size(data,1);
    for row = 1 : numRows
        % Color dates with strong price changes
        if data{row,end} > 0.5
            fgColor = java.awt.Color.green;
        elseif data{row,end} < -0.5
            fgColor = java.awt.Color.red;
        else
            continue
        end
        
        for col = 1 : size(data,2)
            cr.setCellFgColor(row-1,col-1,fgColor)
        end
    end
    jTable.repaint;
    
    %% Color dates with strong volumes
    volumes = [data{:,6}];
    threshold = mean(volumes) + 0.5*std(volumes);
    for row = 1 : numRows
        % Color dates with strong price changes
        if volumes(row) > threshold
            bgColor = java.awt.Color.yellow;
        else
            continue
        end
        
        for col = 1 : size(data,2)
            cr.setCellBgColor(row-1,col-1,bgColor)
        end
    end
    jTable.repaint;

    %% Add table filtering and sorting
    warning off MATLAB:Java:DuplicateClass
    javaaddpath tablefilter-swing-4.4.0.jar
    jFilter = net.coderazzi.filters.gui.TableFilterHeader(jTable); %#ok<NASGU>

    %% Place the data plot in a dedicated tab
    warning off MATLAB:uitabgroup:OldVersion
    warning off MATLAB:uitabgroup:DeprecatedFunction
    warning off MATLAB:uitab:DeprecatedFunction
    hTabGroup = uitabgroup();
    hTab1 = uitab(hTabGroup,'Title','data');
    set(hPanel,'Parent',hTab1)

    hTab2 = uitab(hTabGroup,'Title','Matlab plot');
    hAxes = axes('parent',hTab2);
    allDateNums = datenum(data(:,1),'yyyy-mm-dd');
    closePrices = [data{:,5}]';
    hPlot = plot(hAxes,allDateNums,closePrices);
    set(hTabGroup,'SelectionChangeFcn',{@updateTab,hPanel,[]});

    %% Add a data cursor bar
    hCursorbar = graphics.cursorbar(hPlot); %#ok<NASGU>
    
    %% Set the x-axis ticks to display dates
    datetick(hAxes,'keeplimits');
    drawnow;
    
    %% Now add a 3D pie-chart (JFreeChart)
    try
        hTab3 = uitab(hTabGroup,'Title','JFreeChart plot');
        hPanel3 = uipanel('Parent',hTab3, 'BorderType','none', 'BackgroundColor','w');
        dataset = org.jfree.data.general.DefaultPieDataset;
        percentChanges = [data{:,end}];
        dataset.setValue(java.lang.String('>1% up'),    100*sum(percentChanges>=1)/numRows);
        dataset.setValue(java.lang.String('Positive'),  100*sum(percentChanges>0 & percentChanges<1)/numRows);
        dataset.setValue(java.lang.String('No change'), 100*sum(percentChanges==0)/numRows);
        dataset.setValue(java.lang.String('Negative'),  100*sum(percentChanges<0 & percentChanges>-1)/numRows);
        dataset.setValue(java.lang.String('>1% down'),  100*sum(percentChanges<=-1)/numRows);
        chart3D = org.jfree.chart.ChartFactory.createPieChart3D('Daily market closes', dataset, true, true, false);
        plot3D = [];
        while isempty(plot3D)
            try
                plot3D = chart3D.getPlot;  % an org.jfree.chart.plot.PiePlot3D obj
            catch
                drawnow; pause(0.5);       % wait a bit, to let the graph be updated...
            end
        end
        plot3D.setForegroundAlpha(0.7);    % set transparency level
        plot3D.setDarkerSides(true);       % customization...
        jPanel = org.jfree.chart.ChartPanel(chart3D);
        [jhPanel,hJFreePanel] = javacomponent(jPanel,[20,20,300,300],hPanel3); %#ok<ASGLU>
        set(hJFreePanel, 'Units','normalized', 'Position',[.05,.05,.9,.9], 'Visible','off');
        set(hTabGroup,'SelectionChangeFcn',{@updateTab,hPanel,hJFreePanel});
        drawnow;
    catch
        error('jcommon-1.0.17.jar and jfreechart-1.0.14.jar should be placed in your javaclasspath.txt file')
    end
end

%% Update font
function updateFont(hCombo,eventData,handleToUpdate) %#ok<INUSL>
    value = get(hCombo,'Value');
    strs = get(hCombo,'String');
    htmlStr = strs{value};
    fontName = regexprep(htmlStr,'<.+>','');
    set(handleToUpdate,'FontName',fontName);
end

%% Update table data
function updateDate(jDateChooser,eventData,hTable,data) %#ok<INUSL>
    % prevent multiple callback re-entry
    inCallback = getappdata(hTable,'inDateCallback');
    if isempty(inCallback)
        setappdata(hTable,'inDateCallback',true);

        try
            % Get the new start date
            newDate = jDateChooser.getDate;
            newDateStr = sprintf('%02d/%02d/%04d', newDate.getDate, newDate.getMonth+1, newDate.getYear-100+2000);
            newDateNum = datenum(newDateStr, 'dd/mm/yyyy');

            % Only display dates following the selected date
            allDateNums = datenum(data(:,1),'yyyy-mm-dd');
            okDates = allDateNums >= newDateNum;
            fprintf('Displaying %d dates since %s\n', sum(okDates), newDateStr);

            % Update the main data table
            set(hTable,'Data',data(okDates,:));
        catch
            disp(lasterr);
        end

        % Update the GUI and re-enable callbacks (even in case of error)
        drawnow;
        setappdata(hTable,'inDateCallback',[]);
    end
end

%% Update visibility upon tab selection change
function updateTab(hTabGroup,eventData,hPanel,hJFreePanel) %#ok<INUSL>
    switch eventData.NewValue
        case 1
            set(findall(hPanel),'Visible','on');
            set(hJFreePanel,'Visible','off');
        case 2
            set(findall(hPanel),'Visible','off');
            set(hJFreePanel,'Visible','off');
        case 3
            set(findall(hPanel),'Visible','off');
            set(hJFreePanel,'Visible','on');
    end
end
